package Source::Repository::Chain;

use strict;
use warnings;
use Carp;
use Data::Dumper;

use Source::Repository;#::Interface
our @ISA=qw/Source::Repository/;
our $verbose=1;

sub new {
    my $class=shift;
    my $self={
	# supported: -namelist => filename
	-components => [],
	@_,
    };
    my $components=$self->{-components};
    my @nonempty=grep {ref($_) ne 'Source::Repository::Empty'} @$components;
    $components=$self->{-components}=\@nonempty;
    
    if ($#{$components}<0) {
	# TODO: return empty repo
	die "Source::Repository::Chain: empty";
    }
    if ($#{$components}==0) {
	$components->[0]->{-namelist}//=$self->{-namelist};
	return $components->[0];
    }
    bless $self, $class;

    $self->__load_namelist();# it is checked if $self->{-namelist};

    return $self;
}

# sub push_repository_components {
#     my $self=shift;
#     push @{$self->{-components}}, @_;
# }

#### names should be unique!
sub all_names_ref {
    my ($self)=@_;
    my $names=$self->{-names};
    unless ($names) {
	my %names;
	foreach my $component (@{$self->{-components}}) {
	    map {$names{$_}=1} @{$component->all_names_ref()};
	}
	$self->{-names}=$names=[sort keys(%names)];
    }
    return $names;
}

# TODO: select strategies in new!
sub name2pkg {
    my ($self,$name)=@_;
    my $pkg=$self->name2pkg_max_strategy($name);
    if ($verbose>3) {
	print STDERR "DEBUG: S::R::Chain: name2pkg: $name -> ", &__debug_pkg($pkg),"\n";
    }
    return $pkg;
}

# find first strategy
sub name2pkg_fifo_strategy {
    my ($self,$name)=@_;
    foreach my $component (@{$self->{-components}}) {
	my $pkg=$component->name2pkg($name);
	return $pkg if $pkg;
    }
    return;
}

# find max strategy
sub name2pkg_max_strategy {
    my ($self,$name)=@_;
    my $pkg;
    foreach my $component (@{$self->{-components}}) {
	my $newpkg=$component->name2pkg($name);
	if ($verbose>4) {
	    print STDERR "DEBUG: S::R::Chain: name2pkg_max_strategy: repo=", 
		&__debug_repo($component), " candidate $name -> ", &__debug_pkg($pkg),"\n";
	}
	unless ($pkg) {
	    $pkg=$newpkg;
	} elsif ($newpkg) {
	    $pkg=$newpkg if $component->get_comparator->cmp($newpkg,$pkg)>0;
	}
    }
    return $pkg;
}

sub get_comparator {
    my $self=shift;
    my $comparator=$self->{-comparator};
    return $self->{-components}->[0]->get_comparator();
}

sub is_pkg {
    my ($self,$item)=@_;
    return $self->{-components}->[0]->is_pkg($item);
}

sub file2pkg {
    my ($self,$item)=@_;
    return $self->{-components}->[0]->file2pkg($item);
}

sub __debug_pkg {
    my $pkg=shift;
    return 'undef' if not defined $pkg;
    return $pkg->debugstring();
}

sub __debug_repo {
    my $repo=shift;
    return 'undef' if not defined $repo;
    return '['.$repo->{-reponame}.']='.$repo->{-created_from};
}

1;

__END__
