package Source::Repository::Tarball::CPAN;

use strict;
use warnings;

use File::Basename;
use Source::Package::Tarball::Perl;
use Source::Repository::Tarball;
use Source::Package::Comparators::Raw;

our @ISA=qw/Source::Repository::Tarball/;
our $VERSION=0.003;
our $verbose;

sub get_package_class {
    return 'Source::Package::Tarball::Perl';
}

#------------------------------------------------------
# shared static CPAN maintainance code
#------------------------------------------------------

my %KNOWN_BADVER_DROP=map{$_=>1} qw!
App-NetdiscoX-Web-Plugin-GraphLink
App-NetdiscoX-Web-Plugin-JackLink
App-NetdiscoX-Web-Plugin-JackReport
article-fetcher
Bundle-FinalTest
Bundle-FinalTest2
CGI-Application-Muto-withoutworldwriteables
DAT-TimTen
Data-Dumper-Lazy
Dancer-OO-depfix
DBIx-HTMLView-LATEST
Device-Velleman-K8055-Client
Email-IsFree-CN
File-SimpleDiff
Games-Sudoku-Kubedoku
Geo-GoogleEarth-Document-modules2
IPC-SysV
Lingua-TreeTagger-Filter
link-NCBI
manish-db
Math-Expr-LATEST
Microsoft-Azure-AccessToken
Model3D-WavefrontObject.1.1
Model3D-WavefrontObject
NDFD-Weather-Report
Net-Enum-DNSServer
NET-NETCONF
perl-mlb
Prolog-alpha
RSS-Video-Google
Sendmail-M4.0.29a
Spirit-Of-PallThayer
Spreadsheet-ParseExcel-Assist
Spreadsheet-WriteExcel-WebPivot2
tinyperl-1.0-580-win32
Tk-GraphMan
WebPresence-Profile
XML-DB
!;
my %KNOWN_NOVERSION_DROP=map{$_=>1} qw!
bid
CECALA
CelegansInteractome1.2
CGISession
DataCube
Das
Diamond
EasyDB.0.5b2
examples
FileKGlob
frogbak
FServer
JomonTest
log
lot
Notes
OSS
perlSGML.1997Sep18
PGForth1.0
PGForth1.3
rms
rpn
Saraltest
smg
swd
swig1.1p5
Syringe
theshopbizstore
tic
TimeConvert0.5
UDPServersAndClients
YARN
!;

sub __cpan_load_process_srcpath {
    my ($SRC, $VER, $srcpath, $is_verbose)=@_;
    my $pkg=basename($srcpath);
    return if $pkg=~/\.pm\.(?:gz|bz2)$/;
    my ($module,$ver)=&Source::Package::Tarball::Perl::_cpan_guess_pkg_name_version($pkg);
    if (not defined $module) {
	$pkg=~s/\.(?:tar.gz|tar.bz2|tgz|zip)$//;
	print STDERR "STRANGE2: $pkg\n" if $is_verbose and not $KNOWN_NOVERSION_DROP{$pkg};
	return;
    }

    return if $KNOWN_BADVER_DROP{$module.'-'.$ver};
    eval {
	if (not $SRC->{$module} or &Source::Package::Comparators::Raw::perl_version_cmp($VER->{$module},$ver)<=0) {
		$SRC->{$module}=$srcpath;
		$VER->{$module}=$ver;
	}
    };
    if ($is_verbose and $@) {
	my $msg=$@;
	$msg=~s, at /usr.*$,,;
	print STDERR "pkg=$pkg module=$module oldver=$VER->{$module} newver=$ver: $msg\n";
    }
}

sub _cpan_load_02packages_details_from_FH {
    my ($fh, $is_verbose) =@_;
    my %SRC;
    my %VER;
    while (<$fh>) {
	last if /^\s*$/;
    }
    while (<$fh>) {
	my @pklist=split(/\s+/);
	my $srcpath=$pklist[2];
	&__cpan_load_process_srcpath(\%SRC, \%VER, $srcpath, $is_verbose);
    }
    return (\%SRC,\%VER);
}

sub _cpan_load_01modules_index_html_from_FH {
    my ($fh, $is_verbose) =@_;
    my %SRC;
    my %VER;
    local $_;
    while (<$fh>) {
	chomp;
	&__cpan_load_process_srcpath(\%SRC, \%VER, $_, $is_verbose);
    }
    return (\%SRC,\%VER);
}

1;
