package Source::Repository::Tarball::PyPI;

use strict;
use warnings;

use Source::Shared::Utils::Downloader qw/cached_download/;
use Source::Package::Tarball::Python;
use Source::Repository::Tarball;
our @ISA=qw/Source::Repository::Tarball/;

our $VERSION=0.004;

our $verbose;

sub get_package_class {
    return 'Source::Package::Tarball::Python';
}

### ----------------------------------------------------------

sub pypihref2module_and_trimmed_version {
    my $ref=shift;
    my ($module,$ver)=_pypihref2module_version_pair($ref);
    return ($module,$ver) if not $module or not $ver;
    $ver=&Source::Package::Tarball::Python::_pypi_trim_version($ver);
    return ($module,$ver);
}

sub _pypihref2module_version_pair {
    my ($ref)=@_;
    $ref=~s/\%([0-9A-F])([0-9A-F])/&__mychr($1,$2)/ge;
    my ($module,$ver)=split(m!/!,$ref);
    return ($module,$ver);
}

sub __mychr {
    my ($d1,$d2)=@_;
    return chr(__unhex($d1)*16+__unhex($d2));
}

sub __unhex {
    return $_[0] if $_[0]=~/^[0-9]$/;
    return ord($_[0])+10-ord('A');
}




use File::Basename;
use File::Path qw(make_path remove_tree);
use JSON::XS;
my $PYPI='https://pypi.python.org/pypi/';
my $pypicachedir=$ENV{'HOME'}.'/.cache/pypibuilder/';
sub __get_cached_json_metadata4pypi_module_version_pair {
    my ($module_version_pair)=@_;
    my $file=$pypicachedir.$module_version_pair;
    # https://pypi.python.org/pypi/$module_version_pair/json
    cached_download($PYPI.$module_version_pair.'/json',$pypicachedir);
    if ((-s $file) <3) {
	unlink $file;
	die "Tarball::PyPI: wget $PYPI$module_version_pair/json got an empty file";
    }
    return &_read_file($file);
}
sub get_json_metadata4pypi_module_version_pair {
    my ($module_version_pair)=@_;
    my $metadata_json = &__get_cached_json_metadata4pypi_module_version_pair($module_version_pair);
    my $metadata;
    eval {
	$metadata = decode_json ($metadata_json);
    };
    die "Could not read and parse package metadata\n-----\n$metadata\n------- for $module_version_pair: ",$@ if $@;
    die 'got empty metadata for '.$module_version_pair if ! $metadata;
    return $metadata;
}
sub _read_file {
    my $file=shift;
    my $content = '';
    open my $fh, $file or die $!;
    {
	local $/;
	$content = <$fh>;
    }
    close $fh;
    return $content;
}

sub _pypi_load_pupiurllist_from_FH {
    my $fh=shift;
    my $opt={
	@_,
    };
    my %SRC;
    my %VER;
    my %VEROBJ;
    my $is_verbose=$opt->{-verbose};
    my $allow_all=$opt->{-allow_all};
    while (<$fh>) {
	chomp;
	my $ref=$_;
	&__pypi_load_process_href(\%SRC, \%VER, \%VEROBJ, $ref, $is_verbose,$allow_all);
    }
    return (\%SRC,\%VER);
}

my %KNOWN_NOVERSION_DROP=map{$_=>1} qw!
!;
use Source::Package::Tarball::Python::Version;
sub __pypi_load_process_href {
    my ($SRC, $VER, $VEROBJ, $ref, $is_verbose, $allow_all)=@_;
    my ($module,$ver)=&Source::Repository::Tarball::PyPI::pypihref2module_and_trimmed_version($ref);
    if (not defined $module or not defined $ver) {
	print STDERR "STRANGE2: $ref\n" if $is_verbose and not $KNOWN_NOVERSION_DROP{$module};
	return;
    }
    #return if $KNOWN_BADVER_DROP{$module} and $KNOWN_BADVER_DROP{$module} eq $ver;
    eval {
	my $v2=Source::Package::Tarball::Python::Version->parse($ver);
	if(!$allow_all) {
	    return if $v2->is_prerel();
	    return if $v2->is_devel();
	}
	if (not $SRC->{$module} or &__pycmp($VEROBJ->{$module},$v2)<=0) {
		$SRC->{$module}=$ref;
		$VER->{$module}=$ver;
		$VEROBJ->{$module}=$v2;
	}
    };
    if ($is_verbose and $@) {
	my $msg=$@;
	$msg=~s, at /usr.*$,,;
	print STDERR "ref=$ref module=$module oldver=$VER->{$module} newver=$ver: $msg\n";
    }
}

sub __pycmp {
    my ($v1,$v2)=@_;
    return Source::Package::Tarball::Python::Version::cmp($v1, $v2);
}


1;
